float Brightness(vec3 color)
{
    return max(max(color.r, color.g), color.b);
}


vec3 MedianFliter(vec3 a,vec3 b,vec3 c)
{
    return a + b + c - min(min(a, b), c) - max(max(a, b), c);
     
}

vec3 SafeHDR(vec3 c)
{
    return min(c,vec3(65000.0,65000.0,65000.0));
}

vec4 SafeHDR(vec4 c)
{
    return min(c,vec4(65000.0,65000.0,65000.0,65000.0));
}

vec4 EncodeHDR(vec3 rgb)
{
   // #if USE_RGBM
   // rgb*+1.0/8;
   // float m=max(max(rgb.r,rgb.g),max(rgb.b,1e-6));
  //  m=ceil(m*255)/255;
 //   return flaot4(rgb/m,m);
 //   #else
    return vec4(rgb.x,rgb.y,rgb.z,0.0);
  //  #endif
}



vec3 DeCodeHDR(vec4 rgba)
{
#ifdef USE_RGBM
    return rgba.rgb*rgba.a*8.0;
    #else
    return rgba.rgb;
#endif
}
vec3 DownsampleFliter(Texture2D Etexture,vec2 texelsize,vec2 uv)
{
    vec4  d=texelsize.xyxy*vec4(-1.0,-1.0,+1.0,+1.0);
    vec3 s = vec3(0.0,0.0,0.0);
    s += DeCodeHDR(texture(Etexture,uv + d.xy));
    s += DeCodeHDR(texture(Etexture,uv + d.zy));
    s += DeCodeHDR(texture(Etexture,uv + d.xw));
    s += DeCodeHDR(texture(Etexture,uv + d.zw));
    return s*(1.0/4.0);
}

vec3 DownsampleAntiFlickerFliter(Texture2D Etexture,vec2 texelsize,vec2 uv)
{
    vec4  d=texelsize.xyxy*vec4(-1.0,-1.0,+1.0,+1.0);
    vec3 s1 = DeCodeHDR(texture(Etexture,uv + d.xy));
    vec3 s2 = DeCodeHDR(texture(Etexture,uv + d.zy));
    vec3 s3 = DeCodeHDR(texture(Etexture,uv + d.xw));
    vec3 s4 = DeCodeHDR(texture(Etexture,uv + d.zw));

    float s1w=1.0/(Brightness(s1)+1.0);
    float s2w=1.0/(Brightness(s2)+1.0);
    float s3w=1.0/(Brightness(s3)+1.0);
    float s4w=1.0/(Brightness(s4)+1.0);
    float Invsum=1.0/(s1w + s2w + s3w + s4w);

    return (s1*s1w+s2*s2w+s3*s3w+s4*s4w)*Invsum;
}

vec3 UpsampleFilter(Texture2D Etexture,vec2 texelsize,float samplescale,vec2 uv)
{

    #ifdef HIGHQUALITY
    vec4 d=texelsize.xyxy*vec4(1.0,1.0,-1.0,0.0)*samplescale;
    vec3 s;
   
    s  = DeCodeHDR(texture(Etexture,uv - d.xy));
    s += DeCodeHDR(texture(Etexture,uv - d.wy)) * 2.0;
    s += DeCodeHDR(texture(Etexture,uv -d.zy));

    s += DeCodeHDR(texture(Etexture,uv + d.zw)) * 2.0;
    s += DeCodeHDR(texture(Etexture,uv)) * 4.0;
    s += DeCodeHDR(texture(Etexture,uv + d.xw)) * 2.0;

    s += DeCodeHDR(texture(Etexture,uv + d.zy));
    s += DeCodeHDR(texture(Etexture,uv+ d.wy)) * 2.0;
    s += DeCodeHDR(texture(Etexture,uv+d.xy));

    return s*(1.0/16.0);
    #else
    vec4 d=texelsize.xyxy*vec4(-1.0,-1.0,1.0,1.0)*(samplescale*0.5);
    vec3 s;
    s = DeCodeHDR(texture(Etexture,uv+d.xy));
    s += DeCodeHDR(texture(Etexture,uv+d.zy));
    s += DeCodeHDR(texture(Etexture,uv+d.xw));
    s += DeCodeHDR(texture(Etexture,uv+d.zw));

    return s * (1.0 / 4.0);
#endif

}


vec4 Perfilter(Texture2D Etexture,vec2 texelsize,
float samplescale,vec2 imguv,float PrefilterOffs,vec3 curve,float Threshold)
{

    vec2 uv=imguv+texelsize*PrefilterOffs;
#ifdef ANTIFLICKER
    vec3 d = texelsize.xyx * vec3(1.0, 1.0, 0.0);
    vec4 s0 = SafeHDR(texture(Etexture,uv));
    vec3 s1 = SafeHDR(texture(Etexture,uv  - d.xz).rgb);
    vec3 s2 = SafeHDR(texture(Etexture,uv  + d.xz).rgb);
    vec3 s3 = SafeHDR(texture(Etexture,uv  - d.zy).rgb);
    vec3 s4 = SafeHDR(texture(Etexture,uv  + d.zy).rgb);
    vec3 m = MedianFliter(MedianFliter(s0.rgb, s1, s2), s3, s4);
#else
    vec4 s0=SafeHDR(texture(Etexture,uv));
    vec3 m=s0.rgb;
#endif

    float br=Brightness(m);

    float rq=clamp(br-curve.x,0.0,curve.y);
    rq=curve.z*rq*rq;

    m*=max(rq,br-Threshold)/max(br,1e-5);
    return EncodeHDR(m);
}

vec4 Downsample1(Texture2D Etexture,vec2 texelsize,vec2 uv)
{

#ifdef ANTIFLICKER
    return EncodeHDR(DownsampleAntiFlickerFliter(Etexture,texelsize,uv)) ;
#else
    return EncodeHDR(DownsampleFliter(Etexture,texelsize,uv));
#endif
}

vec4 Downsample2(Texture2D Etexture ,vec2 texelsize,vec2 uv)
{
    return EncodeHDR(DownsampleFliter(Etexture,texelsize,uv));
}


vec4 Upsample(Texture2D Mtexture ,Texture2D Etexture ,
vec2 texelsize,float samplescale,vec2 uv)
{

vec3 base=DeCodeHDR(texture(Etexture,uv));
vec3 blur=UpsampleFilter(Etexture,texelsize,samplescale,uv);
return EncodeHDR(base+blur);

}


vec4 Upsamplefinal(Texture2D Mtexture ,Texture2D Etexture,
vec2 texelsize,float samplescale,vec2 uv,float Intensity)
{
    vec4 base = texture(Mtexture,uv);
    vec3 blur = UpsampleFilter(Etexture,texelsize,samplescale,uv);

    vec3 cout = base.rgb + blur * Intensity;

    return vec4(cout, base.a);
}


